unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, SystemImageList;

type
  TSILDemo = class(TForm)
    Tree: TTreeView;
    List: TListView;
    SmallImages: TdfsSystemImageList;
    LargeImages: TdfsSystemImageList;
    procedure FormCreate(Sender: TObject);
    procedure TreeExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
  private
    function GetFullPath(Node: TTreeNode): string;
    procedure PopulateTree;
    procedure PopulateList(Node: TTreeNode);
    function PopulateNode(Node: TTreeNode): boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SILDemo: TSILDemo;

implementation

{$R *.DFM}

procedure TSILDemo.FormCreate(Sender: TObject);
begin
  PopulateTree;
end;

procedure TSILDemo.PopulateTree;
var
  Node: TTreeNode;
begin
  Tree.Items.BeginUpdate;
  Screen.Cursor := crHourglass;
  try
    Tree.Items.Clear;
    // Add a root node
    Node := Tree.Items.Add(NIL, 'C:');
    Node.ImageIndex := SmallImages.GetImageIndex('C:\', FALSE, FALSE, []);
    Node.SelectedIndex := SmallImages.GetImageIndex('C:\', TRUE, TRUE, []);
    Node.HasChildren := TRUE;
    Node.Data := pointer(1);
  finally
    Screen.Cursor := crDefault;
    Tree.Items.EndUpdate;
  end;
end;


// I'm being lazy here.  I always set HasChildren to TRUE for folders.  This
// results in a "+" for all folders until they are populated or expanded.  If
// I were writing a real app, I would check the folder contents to see if it
// had any subdirectories and only set HasChildren if that was TRUE.
function TSILDemo.PopulateNode(Node: TTreeNode): boolean;
var
  SR: TSearchRec;
  Path: string;
begin
  // If we've already done it, don't do it again.
  if Node.Data = NIL then
  begin
    Result := Node.HasChildren;
    exit;
  end;

  Result := FALSE;
  Tree.Items.BeginUpdate;
  Screen.Cursor := crHourglass;
  try
    Node.Data := NIL; // Flag it so we know we've already done it
    Path := GetFullPath(Node);
    if FindFirst(Path + '*.*', faAnyFile, SR) = 0 then
    begin
      if (SR.Name <> '..') and (SR.Name <> '.') and
         ((SR.Attr and faDirectory) <> 0) then
      begin
        with Tree.Items.AddChild(Node, SR.Name) do
        begin
          Data := pointer(1); // Flag it so we know we haven't done it yet.
          ImageIndex := SmallImages.GetImageIndex(Path + SR.Name, FALSE, FALSE, []);
          SelectedIndex := SmallImages.GetImageIndex(Path + SR.Name, TRUE, TRUE, []);
          // This is kinda cheesy.
          HasChildren := TRUE;
          Result := TRUE;
        end;
      end;

      while FindNext(SR) = 0 do
      begin
        if (SR.Name <> '..') and (SR.Name <> '.') and
           ((SR.Attr and faDirectory) <> 0) then
          with Tree.Items.AddChild(Node, SR.Name) do
          begin
            Data := pointer(1); // Flag it so we know we haven't done it yet.
            ImageIndex := SmallImages.GetImageIndex(Path + SR.Name, FALSE, FALSE, []);
            SelectedIndex := SmallImages.GetImageIndex(Path + SR.Name, TRUE, TRUE, []);
            // This is kinda cheesy.
            HasChildren := TRUE;
            Result := TRUE;
          end;
      end;

      FindClose(SR);
    end;
  finally
    Screen.Cursor := crDefault;
    Tree.Items.EndUpdate;
  end;
end;

procedure TSILDemo.PopulateList(Node: TTreeNode);
var
  SR: TSearchRec;
  Path: string;
  AtLeastOneFolder: boolean;
begin
  AtLeastOneFolder := FALSE;
  List.Items.BeginUpdate;
  Screen.Cursor := crHourglass;
  try
    List.Items.Clear;

    Path := GetFullPath(Node);
    if FindFirst(Path + '*.*', faAnyFile, SR) = 0 then
    begin
      if (SR.Name <> '..') and (SR.Name <> '.') then
      begin
        with List.Items.Add do
        begin
          if ((SR.Attr and faDirectory) <> 0) then
            AtLeastOneFolder := TRUE;
          Caption := SR.Name;
          ImageIndex := SmallImages.GetImageIndex(Path + SR.Name, FALSE, FALSE, []);
        end;
      end;

      while FindNext(SR) = 0 do
      begin
        if (SR.Name <> '..') and (SR.Name <> '.') then
        begin
          if ((SR.Attr and faDirectory) <> 0) then
            AtLeastOneFolder := TRUE;
          with List.Items.Add do
          begin
            Caption := SR.Name;
            ImageIndex := SmallImages.GetImageIndex(Path + SR.Name, FALSE, FALSE, []);
          end;
        end;
      end;

      FindClose(SR);
    end;
  finally
    Screen.Cursor := crDefault;
    List.Items.EndUpdate;
  end;
  Node.HasChildren := AtLeastOneFolder;
end;

procedure TSILDemo.TreeExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
  // This is kinda cheesy.  See PopulateNode
  Node.HasChildren := PopulateNode(Node);
end;

function TSILDemo.GetFullPath(Node: TTreeNode): string;
begin
  Result := '';
  while Node <> NIL do
  begin
    Result := Node.Text + '\' + Result;
    Node := Node.Parent;
  end;
end;

procedure TSILDemo.TreeChange(Sender: TObject; Node: TTreeNode);
begin
  PopulateList(Node);
end;

end.
